package sun.security.pkcs;

import java.security.SecureRandom;
import java.util.List;
import sun.security.timestamp.TimestampToken;
import sun.security.timestamp.TSResponse;
import sun.security.timestamp.TSRequest;
import java.security.MessageDigest;
import sun.security.timestamp.Timestamper;
import java.io.ByteArrayOutputStream;
import sun.security.timestamp.HttpTimestamper;
import java.net.URI;
import sun.security.util.Debug;
import sun.security.x509.X509CertInfo;
import sun.security.x509.X500Name;
import java.util.Vector;
import java.security.SignatureException;
import java.security.NoSuchAlgorithmException;
import sun.security.util.DerEncoder;
import java.util.HashSet;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import java.security.cert.CRLException;
import sun.security.x509.X509CRLImpl;
import java.util.Arrays;
import sun.security.x509.X509CertImpl;
import java.io.ByteArrayInputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import sun.security.util.DerValue;
import java.io.IOException;
import sun.security.util.DerInputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import java.security.Principal;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import sun.security.x509.AlgorithmId;
import java.math.BigInteger;
import sun.security.util.ObjectIdentifier;

public class PKCS7
{
    private ObjectIdentifier contentType;
    private BigInteger version;
    private AlgorithmId[] digestAlgorithmIds;
    private ContentInfo contentInfo;
    private X509Certificate[] certificates;
    private X509CRL[] crls;
    private SignerInfo[] signerInfos;
    private boolean oldStyle;
    private Principal[] certIssuerNames;
    private static final String KP_TIMESTAMPING_OID = "1.3.6.1.5.5.7.3.8";
    private static final String EXTENDED_KEY_USAGE_OID = "2.5.29.37";
    
    public PKCS7(InputStream in) throws ParsingException, IOException {
        this.version = null;
        this.digestAlgorithmIds = null;
        this.contentInfo = null;
        this.certificates = null;
        this.crls = null;
        this.signerInfos = null;
        this.oldStyle = false;
        DataInputStream dataInputStream = new DataInputStream(in);
        byte[] array = new byte[dataInputStream.available()];
        dataInputStream.readFully(array);
        this.parse(new DerInputStream(array));
    }
    
    public PKCS7(DerInputStream derInputStream) throws ParsingException {
        this.version = null;
        this.digestAlgorithmIds = null;
        this.contentInfo = null;
        this.certificates = null;
        this.crls = null;
        this.signerInfos = null;
        this.oldStyle = false;
        this.parse(derInputStream);
    }
    
    public PKCS7(byte[] data) throws ParsingException {
        this.version = null;
        this.digestAlgorithmIds = null;
        this.contentInfo = null;
        this.certificates = null;
        this.crls = null;
        this.signerInfos = null;
        this.oldStyle = false;
        try {
            this.parse(new DerInputStream(data));
        }
        catch (IOException cause) {
            ParsingException ex = new ParsingException("Unable to parse the encoded bytes");
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private void parse(DerInputStream derInputStream) throws ParsingException {
        try {
            derInputStream.mark(derInputStream.available());
            this.parse(derInputStream, false);
        }
        catch (IOException cause) {
            try {
                derInputStream.reset();
                this.parse(derInputStream, true);
                this.oldStyle = true;
            }
            catch (IOException exception) {
                ParsingException ex = new ParsingException(exception.getMessage());
                ex.initCause(cause);
                ex.addSuppressed(exception);
                throw ex;
            }
        }
    }
    
    private void parse(DerInputStream derInputStream, boolean b) throws IOException {
        this.contentInfo = new ContentInfo(derInputStream, b);
        this.contentType = this.contentInfo.contentType;
        DerValue content = this.contentInfo.getContent();
        if (this.contentType.equals(ContentInfo.SIGNED_DATA_OID)) {
            this.parseSignedData(content);
        }
        else if (this.contentType.equals(ContentInfo.OLD_SIGNED_DATA_OID)) {
            this.parseOldSignedData(content);
        }
        else if (this.contentType.equals(ContentInfo.NETSCAPE_CERT_SEQUENCE_OID)) {
            this.parseNetscapeCertChain(content);
        }
        else {
            throw new ParsingException("content type " + this.contentType + " not supported.");
        }
    }
    
    public PKCS7(AlgorithmId[] digestAlgorithmIds, ContentInfo contentInfo, X509Certificate[] certificates, X509CRL[] crls, SignerInfo[] signerInfos) {
        this.version = null;
        this.digestAlgorithmIds = null;
        this.contentInfo = null;
        this.certificates = null;
        this.crls = null;
        this.signerInfos = null;
        this.oldStyle = false;
        this.version = BigInteger.ONE;
        this.digestAlgorithmIds = digestAlgorithmIds;
        this.contentInfo = contentInfo;
        this.certificates = certificates;
        this.crls = crls;
        this.signerInfos = signerInfos;
    }
    
    public PKCS7(AlgorithmId[] array, ContentInfo contentInfo, X509Certificate[] array2, SignerInfo[] array3) {
        this(array, contentInfo, array2, null, array3);
    }
    
    private void parseNetscapeCertChain(DerValue derValue) throws ParsingException, IOException {
        DerValue[] sequence = new DerInputStream(derValue.toByteArray()).getSequence(2);
        this.certificates = new X509Certificate[sequence.length];
        CertificateFactory instance = null;
        try {
            instance = CertificateFactory.getInstance("X.509");
        }
        catch (CertificateException ex) {}
        for (int i = 0; i < sequence.length; ++i) {
            Object inStream = null;
            try {
                if (instance == null) {
                    this.certificates[i] = new X509CertImpl(sequence[i]);
                }
                else {
                    inStream = new ByteArrayInputStream(sequence[i].toByteArray());
                    this.certificates[i] = (X509Certificate)instance.generateCertificate(inStream);
                    inStream.close();
                    inStream = null;
                }
            }
            catch (CertificateException cause) {
                ParsingException o = new ParsingException(cause.getMessage());
                o.initCause(cause);
                throw o;
            }
            catch (IOException cause2) {
                ParsingException o2 = new ParsingException(cause2.getMessage());
                o2.initCause(cause2);
                throw o2;
            }
            finally {
                if (inStream != null) {
                    inStream.close();
                }
            }
        }
    }
    
    private void parseSignedData(DerValue derValue) throws ParsingException, IOException {
        DerInputStream derInputStream = derValue.toDerInputStream();
        this.version = derInputStream.getBigInteger();
        DerValue[] set = derInputStream.getSet(1);
        int length = set.length;
        this.digestAlgorithmIds = new AlgorithmId[length];
        try {
            for (int i = 0; i < length; ++i) {
                this.digestAlgorithmIds[i] = AlgorithmId.parse(set[i]);
            }
        }
        catch (IOException cause) {
            ParsingException ex = new ParsingException("Error parsing digest AlgorithmId IDs: " + cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
        this.contentInfo = new ContentInfo(derInputStream);
        CertificateFactory instance = null;
        try {
            instance = CertificateFactory.getInstance("X.509");
        }
        catch (CertificateException ex2) {}
        if ((byte)derInputStream.peekByte() == -96) {
            DerValue[] set2 = derInputStream.getSet(2, true);
            int length2 = set2.length;
            this.certificates = new X509Certificate[length2];
            int newLength = 0;
            for (int j = 0; j < length2; ++j) {
                Object inStream = null;
                try {
                    if (set2[j].getTag() == 48) {
                        if (instance == null) {
                            this.certificates[newLength] = new X509CertImpl(set2[j]);
                        }
                        else {
                            inStream = new ByteArrayInputStream(set2[j].toByteArray());
                            this.certificates[newLength] = (X509Certificate)instance.generateCertificate(inStream);
                            inStream.close();
                            inStream = null;
                        }
                        ++newLength;
                    }
                }
                catch (CertificateException cause2) {
                    ParsingException o = new ParsingException(cause2.getMessage());
                    o.initCause(cause2);
                    throw o;
                }
                catch (IOException cause3) {
                    ParsingException o2 = new ParsingException(cause3.getMessage());
                    o2.initCause(cause3);
                    throw o2;
                }
                finally {
                    if (inStream != null) {
                        inStream.close();
                    }
                }
            }
            if (newLength != length2) {
                this.certificates = (X509Certificate[])Arrays.copyOf(this.certificates, newLength);
            }
        }
        if ((byte)derInputStream.peekByte() == -95) {
            DerValue[] set3 = derInputStream.getSet(1, true);
            int length3 = set3.length;
            this.crls = new X509CRL[length3];
            for (int k = 0; k < length3; ++k) {
                Object inStream2 = null;
                try {
                    if (instance == null) {
                        this.crls[k] = new X509CRLImpl(set3[k]);
                    }
                    else {
                        inStream2 = new ByteArrayInputStream(set3[k].toByteArray());
                        this.crls[k] = (X509CRL)instance.generateCRL(inStream2);
                        inStream2.close();
                        inStream2 = null;
                    }
                }
                catch (CRLException cause4) {
                    ParsingException o3 = new ParsingException(cause4.getMessage());
                    o3.initCause(cause4);
                    throw o3;
                }
                finally {
                    if (inStream2 != null) {
                        inStream2.close();
                    }
                }
            }
        }
        DerValue[] set4 = derInputStream.getSet(1);
        int length4 = set4.length;
        this.signerInfos = new SignerInfo[length4];
        for (int l = 0; l < length4; ++l) {
            this.signerInfos[l] = new SignerInfo(set4[l].toDerInputStream());
        }
    }
    
    private void parseOldSignedData(DerValue derValue) throws ParsingException, IOException {
        DerInputStream derInputStream = derValue.toDerInputStream();
        this.version = derInputStream.getBigInteger();
        DerValue[] set = derInputStream.getSet(1);
        int length = set.length;
        this.digestAlgorithmIds = new AlgorithmId[length];
        try {
            for (int i = 0; i < length; ++i) {
                this.digestAlgorithmIds[i] = AlgorithmId.parse(set[i]);
            }
        }
        catch (IOException ex) {
            throw new ParsingException("Error parsing digest AlgorithmId IDs");
        }
        this.contentInfo = new ContentInfo(derInputStream, true);
        CertificateFactory instance = null;
        try {
            instance = CertificateFactory.getInstance("X.509");
        }
        catch (CertificateException ex2) {}
        DerValue[] set2 = derInputStream.getSet(2);
        int length2 = set2.length;
        this.certificates = new X509Certificate[length2];
        for (int j = 0; j < length2; ++j) {
            Object inStream = null;
            try {
                if (instance == null) {
                    this.certificates[j] = new X509CertImpl(set2[j]);
                }
                else {
                    inStream = new ByteArrayInputStream(set2[j].toByteArray());
                    this.certificates[j] = (X509Certificate)instance.generateCertificate(inStream);
                    inStream.close();
                    inStream = null;
                }
            }
            catch (CertificateException cause) {
                ParsingException o = new ParsingException(cause.getMessage());
                o.initCause(cause);
                throw o;
            }
            catch (IOException cause2) {
                ParsingException o2 = new ParsingException(cause2.getMessage());
                o2.initCause(cause2);
                throw o2;
            }
            finally {
                if (inStream != null) {
                    inStream.close();
                }
            }
        }
        derInputStream.getSet(0);
        DerValue[] set3 = derInputStream.getSet(1);
        int length3 = set3.length;
        this.signerInfos = new SignerInfo[length3];
        for (int k = 0; k < length3; ++k) {
            this.signerInfos[k] = new SignerInfo(set3[k].toDerInputStream(), true);
        }
    }
    
    public void encodeSignedData(OutputStream outputStream) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        this.encodeSignedData(derOutputStream);
        outputStream.write(derOutputStream.toByteArray());
    }
    
    public void encodeSignedData(DerOutputStream derOutputStream) throws IOException {
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.putInteger(this.version);
        derOutputStream2.putOrderedSetOf((byte)49, this.digestAlgorithmIds);
        this.contentInfo.encode(derOutputStream2);
        if (this.certificates != null && this.certificates.length != 0) {
            X509CertImpl[] array = new X509CertImpl[this.certificates.length];
            for (int i = 0; i < this.certificates.length; ++i) {
                if ((this.certificates[i] instanceof X509CertImpl)) {
                    array[i] = (X509CertImpl)this.certificates[i];
                }
                else {
                    try {
                        array[i] = new X509CertImpl(this.certificates[i].getEncoded());
                    }
                    catch (CertificateException cause) {
                        throw new IOException(cause);
                    }
                }
            }
            derOutputStream2.putOrderedSetOf((byte)(-96), array);
        }
        if (this.crls != null && this.crls.length != 0) {
            HashSet o = new HashSet(this.crls.length);
            for (X509CRL x509CRL : this.crls) {
                if ((x509CRL instanceof X509CRLImpl)) {
                    o.add((X509CRLImpl)x509CRL);
                }
                else {
                    try {
                        o.add(new X509CRLImpl(x509CRL.getEncoded()));
                    }
                    catch (CRLException cause2) {
                        throw new IOException(cause2);
                    }
                }
            }
            derOutputStream2.putOrderedSetOf((byte)(-95), (DerEncoder[])o.toArray(new X509CRLImpl[o.size()]));
        }
        derOutputStream2.putOrderedSetOf((byte)49, this.signerInfos);
        new ContentInfo(ContentInfo.SIGNED_DATA_OID, new DerValue((byte)48, derOutputStream2.toByteArray())).encode(derOutputStream);
    }
    
    public SignerInfo verify(SignerInfo signerInfo, byte[] array) throws NoSuchAlgorithmException, SignatureException {
        return signerInfo.verify(this, array);
    }
    
    public SignerInfo[] verify(byte[] array) throws NoSuchAlgorithmException, SignatureException {
        Vector<SignerInfo> vector = new Vector();
        for (int i = 0; i < this.signerInfos.length; ++i) {
            SignerInfo verify = this.verify(this.signerInfos[i], array);
            if (verify != null) {
                vector.addElement(verify);
            }
        }
        if (!vector.isEmpty()) {
            SignerInfo[] anArray = new SignerInfo[vector.size()];
            vector.copyInto(anArray);
            return anArray;
        }
        return null;
    }
    
    public SignerInfo[] verify() throws NoSuchAlgorithmException, SignatureException {
        return this.verify((byte[])null);
    }
    
    public BigInteger getVersion() {
        return this.version;
    }
    
    public AlgorithmId[] getDigestAlgorithmIds() {
        return this.digestAlgorithmIds;
    }
    
    public ContentInfo getContentInfo() {
        return this.contentInfo;
    }
    
    public X509Certificate[] getCertificates() {
        if (this.certificates != null) {
            return (X509Certificate[])this.certificates.clone();
        }
        return null;
    }
    
    public X509CRL[] getCRLs() {
        if (this.crls != null) {
            return (X509CRL[])this.crls.clone();
        }
        return null;
    }
    
    public SignerInfo[] getSignerInfos() {
        return this.signerInfos;
    }
    
    public X509Certificate getCertificate(BigInteger bigInteger, X500Name x500Name) {
        if (this.certificates != null) {
            if (this.certIssuerNames == null) {
                this.populateCertIssuerNames();
            }
            int i = 0;
            while (i < this.certificates.length) {
                X509Certificate x509Certificate = this.certificates[i];
                if (bigInteger.equals(x509Certificate.getSerialNumber()) && x500Name.equals(this.certIssuerNames[i])) {
                    return x509Certificate;
                }
                ++i;
            }
        }
        return null;
    }
    
    private void populateCertIssuerNames() {
        if (this.certificates == null) {
            return;
        }
        this.certIssuerNames = new Principal[this.certificates.length];
        Principal issuerDN;
        for (int i = 0; i < this.certificates.length; this.certIssuerNames[i] = issuerDN, ++i) {
            X509Certificate x509Certificate = this.certificates[i];
            issuerDN = x509Certificate.getIssuerDN();
            if (!(issuerDN instanceof X500Name)) {
                try {
                    issuerDN = (Principal)new X509CertInfo(x509Certificate.getTBSCertificate()).get("issuer.dname");
                }
                catch (Exception ex) {}
            }
        }
        return;
    }
    
    @Override
    public String toString() {
        String str = "" + this.contentInfo + "\n";
        if (this.version != null) {
            str = str + "PKCS7 :: version: " + Debug.toHexString(this.version) + "\n";
        }
        if (this.digestAlgorithmIds != null) {
            str += "PKCS7 :: digest AlgorithmIds: \n";
            for (int i = 0; i < this.digestAlgorithmIds.length; ++i) {
                str = str + "\t" + this.digestAlgorithmIds[i] + "\n";
            }
        }
        if (this.certificates != null) {
            str += "PKCS7 :: certificates: \n";
            for (int j = 0; j < this.certificates.length; ++j) {
                str = str + "\t" + j + ".   " + this.certificates[j] + "\n";
            }
        }
        if (this.crls != null) {
            str += "PKCS7 :: crls: \n";
            for (int k = 0; k < this.crls.length; ++k) {
                str = str + "\t" + k + ".   " + this.crls[k] + "\n";
            }
        }
        if (this.signerInfos != null) {
            str += "PKCS7 :: signer infos: \n";
            for (int l = 0; l < this.signerInfos.length; ++l) {
                str = str + "\t" + l + ".  " + this.signerInfos[l] + "\n";
            }
            return str;
        }
        return str;
    }
    
    public boolean isOldStyle() {
        return this.oldStyle;
    }
    
    public static byte[] generateSignedData(byte[] array, X509Certificate[] array2, byte[] array3, String s, URI tsaURI, String s2, String s3) throws CertificateException, IOException, NoSuchAlgorithmException {
        PKCS9Attributes pkcs9Attributes = null;
        if (tsaURI != null) {
            pkcs9Attributes = new PKCS9Attributes(new PKCS9Attribute[] { new PKCS9Attribute("SignatureTimestampToken", generateTimestampToken(new HttpTimestamper(tsaURI), s2, s3, array)) });
        }
        SignerInfo o = new SignerInfo(X500Name.asX500Name(array2[0].getIssuerX500Principal()), array2[0].getSerialNumber(), AlgorithmId.get(AlgorithmId.getDigAlgFromSigAlg(s)), null, AlgorithmId.get(AlgorithmId.getEncAlgFromSigAlg(s)), array, pkcs9Attributes);
        PKCS7 o2 = new PKCS7(new AlgorithmId[] { o.getDigestAlgorithmId() }, (array3 == null) ? new ContentInfo(ContentInfo.DATA_OID, null) : new ContentInfo(array3), array2, new SignerInfo[] { o });
        ByteArrayOutputStream o3 = new ByteArrayOutputStream();
        o2.encodeSignedData(o3);
        return o3.toByteArray();
    }
    
    private static byte[] generateTimestampToken(Timestamper timestamper, String tsaPolicyID, String str, byte[] toBeTimeStamped) throws IOException, CertificateException {
        TSRequest tsRequest;
        try {
            tsRequest = new TSRequest(tsaPolicyID, toBeTimeStamped, MessageDigest.getInstance(str));
        }
        catch (NoSuchAlgorithmException cause) {
            throw new IllegalArgumentException(cause);
        }
        BigInteger bigInteger = null;
        if (PKCS7.SecureRandomHolder.RANDOM != null) {
            bigInteger = new BigInteger(64, PKCS7.SecureRandomHolder.RANDOM);
            tsRequest.setNonce(bigInteger);
        }
        tsRequest.requestCertificate(true);
        TSResponse generateTimestamp = timestamper.generateTimestamp(tsRequest);
        int statusCode = generateTimestamp.getStatusCode();
        if (statusCode != 0 && statusCode != 1) {
            throw new IOException("Error generating timestamp: " + generateTimestamp.getStatusCodeAsText() + " " + generateTimestamp.getFailureCodeAsText());
        }
        if (tsaPolicyID != null && !tsaPolicyID.equals(generateTimestamp.getTimestampToken().getPolicyID())) {
            throw new IOException("TSAPolicyID changed in timestamp token");
        }
        PKCS7 token = generateTimestamp.getToken();
        TimestampToken timestampToken = generateTimestamp.getTimestampToken();
        try {
            if (!timestampToken.getHashAlgorithm().equals(AlgorithmId.get(str))) {
                throw new IOException("Digest algorithm not " + str + " in timestamp token");
            }
        }
        catch (NoSuchAlgorithmException ex) {
            throw new IllegalArgumentException();
        }
        if (!MessageDigest.isEqual(timestampToken.getHashedMessage(), tsRequest.getHashedMessage())) {
            throw new IOException("Digest octets changed in timestamp token");
        }
        BigInteger nonce = timestampToken.getNonce();
        if (nonce == null && bigInteger != null) {
            throw new IOException("Nonce missing in timestamp token");
        }
        if (nonce != null && !nonce.equals(bigInteger)) {
            throw new IOException("Nonce changed in timestamp token");
        }
        SignerInfo[] signerInfos = token.getSignerInfos();
        int length = signerInfos.length;
        int i = 0;
        while (i < length) {
            X509Certificate certificate = signerInfos[i].getCertificate(token);
            if (certificate == null) {
                throw new CertificateException("Certificate not included in timestamp token");
            }
            if (!certificate.getCriticalExtensionOIDs().contains("2.5.29.37")) {
                throw new CertificateException("Certificate is not valid for timestamping");
            }
            List<String> extendedKeyUsage = certificate.getExtendedKeyUsage();
            if (extendedKeyUsage == null || !extendedKeyUsage.contains("1.3.6.1.5.5.7.3.8")) {
                throw new CertificateException("Certificate is not valid for timestamping");
            }
            ++i;
        }
        return generateTimestamp.getEncodedToken();
    }
    
    private static class SecureRandomHolder
    {
        static final SecureRandom RANDOM;
        
        static {
            SecureRandom instance = null;
            try {
                instance = SecureRandom.getInstance("SHA1PRNG");
            }
            catch (NoSuchAlgorithmException ex) {}
            RANDOM = instance;
        }
    }
}
